@@ -66,11 +66,10 @@ from z3 import (
     IsMember,
     Not,
     SetAdd,
-    Solver,
-    sat,
     simplify,
 )
 
+
 # Standard imports
 from model_checker.utils import (
     not_implemented_string,
@@ -139,7 +138,9 @@ class SemanticDefaults:
     """
 
     def __init__(self, combined_settings):
-
+        # Reset any global state to avoid cross-example interference
+        self._reset_global_state()
+        
         # Store the name
         self.name = self.__class__.__name__
 
@@ -165,6 +166,34 @@ class SemanticDefaults:
         # Define invalidity conditions
         self.premise_behavior = None
         self.conclusion_behavior = None
+        
+    def _reset_global_state(self):
+        """Reset any global state that could cause interference between examples.
+        
+        Following the fail-fast philosophy, this method explicitly resets all cached
+        state that could lead to non-deterministic behavior between examples.
+        
+        Subclasses MUST override this method and call super()._reset_global_state()
+        to ensure proper cleanup of both shared and theory-specific resources.
+        
+        Example for subclasses:
+        
+        def _reset_global_state(self):
+            # Always call parent implementation first
+            super()._reset_global_state()
+            
+            # Reset theory-specific caches
+            self._theory_specific_cache = {}
+            
+            # Clear any references to model structures
+            if hasattr(self, 'model_structure'):
+                delattr(self, 'model_structure')
+                
+            # Reset mutable data structures but preserve immutable definitions
+            self.data_cache = {}
+        """
+        # Reset general caches
+        self._cached_values = {}
 
     def fusion(self, bit_s, bit_t):
         """Performs the fusion operation on two bit vectors.
@@ -414,7 +443,7 @@ class ModelConstraints:
     
     Attributes:
         settings (dict): Configuration settings for model generation
-        syntax (Syntax): The syntactic representation of the argument
+        syntax (Syntax): The syntactic representation of the argument # TODO: Not sure this is right?
         semantics (Semantics): The semantic theory being used
         proposition_class (class): The class used to create propositions
         sentences (dict): All sentences in the argument
@@ -608,6 +637,10 @@ class ModelDefaults:
     Specific theories extend this class to implement theory-specific model structures
     with additional functionality and visualization capabilities.
     
+    The class implements careful isolation between examples to ensure that each example
+    is solved independently with its own Z3 solver instance. This prevents state leakage
+    between examples that could affect solver behavior, performance, or results.
+    
     Attributes:
         model_constraints (ModelConstraints): The constraints to satisfy in the model
         settings (dict): Configuration settings for model generation
@@ -633,8 +666,8 @@ class ModelDefaults:
         self.RESET = "\033[0m"
         self.WHITE = self.COLORS["default"]
 
-        # TODO: is this still needed?
-        self.constraint_dict = {} # hopefully temporary, for unsat_core
+        # Dictionary for tracking constraints for unsat_core
+        self.constraint_dict = {}
 
         # Store arguments
         self.model_constraints = model_constraints
@@ -660,19 +693,24 @@ class ModelDefaults:
         self.settings = self.model_constraints.settings
 
         # Initialize Z3 attributes
-        self.solver = None # TODO: still needed?
+        self.solver = None
+        self.stored_solver = None
         self.timeout = None
         self.z3_model = None
         self.unsat_core = None
         self.z3_model_status = None
         self.z3_model_runtime = None
+        
+        # Reset Z3 context before creating a new solver to ensure isolation
+        from model_checker.utils import Z3ContextManager
+        Z3ContextManager.reset_context()
 
         # Solve Z3 constraints and store results
         solver_results = self.solve(self.model_constraints, self.max_time)
         self._process_solver_results(solver_results)
 
         # Exit early if no valid model was found
-        if self.timeout or self.z3_model is None:
+        if self.timeout or self.z3_model is None: # M: does this do anything?
             return
 
     def _process_solver_results(self, solver_results):
@@ -718,7 +756,12 @@ class ModelDefaults:
             Constraints are added using assert_and_track() to enable unsat core generation
             when constraints are unsatisfiable.
         """
+        from z3 import Solver
         solver = Solver()
+            
+        # Clear the constraint dict to prevent cross-example contamination
+        self.constraint_dict = {}
+            
         constraint_groups = [
             (model_constraints.frame_constraints, "frame"),
             (model_constraints.model_constraints, "model"), 
@@ -750,12 +793,48 @@ class ModelDefaults:
         runtime = round(time.time() - start_time, 4)
         return is_timeout, model_or_core, is_satisfiable, runtime
 
+    def _cleanup_solver_resources(self):
+        """Explicitly clean up Z3 resources to ensure complete isolation between examples.
+        
+        This method is crucial for preventing state leakage between different examples.
+        It explicitly removes references to Z3 objects to ensure that Z3 resources 
+        are properly released. This helps prevent unexpected interactions between 
+        examples that could affect solving behavior or performance.
+        
+        IMPORTANT: This is a core part of the solver state isolation system. It should be
+        called in the finally block of solve() and related methods to ensure resources
+        are always released, even if exceptions occur:
+        
+        ```python
+        try:
+            # Setup and run solver
+            # ...
+        except Exception as e:
+            # Handle exceptions
+            # ...
+        finally:
+            # Always clean up
+            self._cleanup_solver_resources()
+        ```
+        
+        For comprehensive information on Z3 solver state management, see:
+        - theory_lib/specs/solvers.md
+        - ../../docs/DEVELOPMENT.md section on Z3 Solver State Management
+        """
+        # Remove references to solver and model
+        self.solver = None
+        self.z3_model = None
+        
+        # Clear the context reference (if it exists)
+        if hasattr(self, 'example_context'):
+            self.example_context = None
+    
     def solve(self, model_constraints, max_time):
         """Uses the Z3 solver to find a model satisfying the given constraints.
         
-        This method sets up the Z3 solver with all the constraints derived from
-        the premises, conclusions, and frame conditions, then attempts to find
-        a satisfying assignment that represents a valid model.
+        Creates a completely fresh Z3 context for each example to ensure
+        proper isolation and deterministic behavior regardless of which
+        examples were run previously.
         
         Args:
             model_constraints (ModelConstraints): The logical constraints to solve
@@ -768,8 +847,15 @@ class ModelDefaults:
             - If the constraints are unsatisfiable, returns the unsatisfiable core
             - If solving times out, sets the timeout flag but still returns partial results
         """
+        # Import z3
+        import z3
+        
+        # Create a new solver
+        self.solver = z3.Solver()
+        self.stored_solver = self.solver
 
         try:
+            # Set up the solver with the constraints
             self.solver = self._setup_solver(model_constraints)
 
             # Set timeout and solve
@@ -778,7 +864,7 @@ class ModelDefaults:
             result = self.solver.check()
             
             # Handle different solver outcomes
-            if result == sat:
+            if result == z3.sat:
                 return self._create_result(False, self.solver.model(), True, start_time)
             
             if self.solver.reason_unknown() == "timeout":
@@ -787,8 +873,12 @@ class ModelDefaults:
             return self._create_result(False, self.solver.unsat_core(), False, start_time)
             
         except RuntimeError as e:
-            print(f"An error occurred while running `solve_constraints()`: {e}")
+            print(f"An error occurred during solving: {e}")
             return True, None, False, None
+        finally:
+            # Ensure proper cleanup to prevent any possible state leakage
+            self._cleanup_solver_resources()
+            # NOTE: if a new solver is used every time, why do we need to clean the solver up?
     
     def re_solve(self):
         """Re-solve the existing model constraints with the current solver state.
@@ -808,6 +898,7 @@ class ModelDefaults:
             RuntimeError: If solver encounters an error during execution
             AssertionError: If solver instance doesn't exist
         """
+        import z3
 
         try:
             assert self.solver is not None
@@ -818,7 +909,7 @@ class ModelDefaults:
             result = self.solver.check()
             
             # Handle different solver outcomes
-            if result == sat:
+            if result == z3.sat:
                 return self._create_result(False, self.solver.model(), True, start_time)
             
             if self.solver.reason_unknown() == "timeout":
@@ -1108,6 +1199,7 @@ class ModelDefaults:
         # Default implementation returns None, meaning use basic difference detection
         return None
         
+    # TODO: Is this needed or can it be removed? Evaluate when working on the iterator tool
     def print_model_differences(self, output=sys.stdout):
         """Prints differences between this model and the previous one.
         
@@ -1118,7 +1210,8 @@ class ModelDefaults:
             output (file, optional): Output stream to write to. Defaults to sys.stdout.
         """
         if not hasattr(self, 'model_differences') or not self.model_differences:
-            print("No previous model to compare with.", file=output)
+            # print("No previous model to compare with.", file=output)
+            # print(f"DEBUG: hasattr model_differences = {hasattr(self, 'model_differences')}, value = {getattr(self, 'model_differences', 'N/A')}", file=output)
             return
         
         print("\n=== DIFFERENCES FROM PREVIOUS MODEL ===\n", file=output)
@@ -1128,7 +1221,13 @@ class ModelDefaults:
         if letter_diffs:
             print("Sentence Letter Changes:", file=output)
             for letter, values in letter_diffs.items():
-                print(f"  {letter}: {values['old']} → {values['new']}", file=output)
+                try:
+                    if 'old' in values and 'new' in values:
+                        print(f"  {letter}: {values['old']} → {values['new']}", file=output)
+                    else:
+                        print(f"  {letter}: changed from previous model", file=output)
+                except (KeyError, TypeError) as e:
+                    print(f"  {letter}: changed from previous model", file=output)
             print("", file=output)
             
         # Print semantic function differences
@@ -1138,7 +1237,13 @@ class ModelDefaults:
             for func_name, values in func_diffs.items():
                 print(f"  {func_name}:", file=output)
                 for input_val, change in values.items():
-                    print(f"    Input {input_val}: {change['old']} → {change['new']}", file=output)
+                    try:
+                        if 'old' in change and 'new' in change:
+                            print(f"    Input {input_val}: {change['old']} → {change['new']}", file=output)
+                        else:
+                            print(f"    Input {input_val}: changed from previous model", file=output)
+                    except (KeyError, TypeError) as e:
+                        print(f"    Input {input_val}: changed from previous model", file=output)
             print("", file=output)
             
         # Print model structure differences
@@ -1146,7 +1251,13 @@ class ModelDefaults:
         if struct_diffs:
             print("Model Structure Changes:", file=output)
             for component, values in struct_diffs.items():
-                print(f"  {component}: {values['old']} → {values['new']}", file=output)
+                try:
+                    if 'old' in values and 'new' in values:
+                        print(f"  {component}: {values['old']} → {values['new']}", file=output)
+                    else:
+                        print(f"  {component}: changed from previous model", file=output)
+                except (KeyError, TypeError) as e:
+                    print(f"  {component}: changed from previous model", file=output)
             print("", file=output)
         
         # Print structural metrics if available
@@ -1237,5 +1348,5 @@ class ModelDefaults:
     def _print_runtime_footer(self, output):
         """Print Z3 runtime and separator footer."""
         print(f"\nZ3 Run Time: {self.z3_model_runtime} seconds", file=output)
-        print(f"\n{'='*40}\n", file=output)
+        print(f"\n{'='*40}", file=output)
 
