@@ -45,6 +45,19 @@ from z3 import Const, DeclareSort, ExprRef
 
 AtomSort = DeclareSort("AtomSort")
 
+def AtomVal(i):
+    """Create a constant of AtomSort with the given index.
+    
+    This helper function creates atomic proposition values for use in semantics.
+    
+    Args:
+        i (int): The index for the atomic proposition
+        
+    Returns:
+        Const: A Z3 constant of AtomSort
+    """
+    return Const(f"AtomSort!val!{i}", AtomSort)
+
 class Sentence:
     """Represents a logical sentence with support for both infix and prefix notation.
     
@@ -359,16 +372,6 @@ class Operator:
         model_structure = proposition.model_structure
         N = proposition.N
 
-        # TODO: make more deterministic
-        # Handle both world_id and traditional world approaches for compatibility
-        # In the bimodal theory, we're now using world_id but the base code expects world
-        if "world" in eval_point:
-            current_world_id = eval_point["world"]
-            if "world" not in eval_point and hasattr(model_structure, "get_world_array"):
-                eval_point["world"] = model_structure.get_world_array(current_world_id)
-        else:
-            current_world = eval_point.get("world")
-
         proposition.print_proposition(eval_point, indent_num, use_colors)
         indent_num += 1
 
@@ -768,9 +771,6 @@ class Syntax:
             if infix_sentence in self.all_sentences.keys():
                 return self.all_sentences[infix_sentence]
             sentence = Sentence(infix_sentence)
-            # TODO: confirm not needed
-            # if sentence.original_operator:
-            #     self.operators_used.append(sentence.original_operator)
             self.all_sentences[sentence.name] = sentence
             if sentence.original_arguments is None:
                 if sentence.name.isalnum():
@@ -801,10 +801,6 @@ class Syntax:
                 - Updates sentence.original_type with operator classes
                 - Updates sentence.arguments for defined operators
             """
-            # TODO: confirm not needed with derived operators
-            # if sentence.original_arguments:
-            #     for argument in sentence.original_arguments:
-            #         initialize_types(argument)
             sentence.update_types(self.operator_collection)
             if sentence.arguments: # NOTE: must happen after arguments are stored
                 sentence_arguments = []
@@ -816,9 +812,6 @@ class Syntax:
 
         sentence_objects = []
         for infix_sent in infix_sentences:
-            # TODO: this check/continue leads to errors
-            # if infix_sent in self.all_sentences.keys():
-            #     continue
             sentence = build_sentence(infix_sent)
             initialize_types(sentence)
             sentence_objects.append(sentence)
