@@ -40,6 +40,10 @@ class BimodalSemantics(SemanticDefaults):
         'max_time': 1,
         # Whether a model is expected or not (used for unit testing)
         'expectation': True,
+        # Number of model iterations to generate
+        'iterate': 1,
+        # Timeout for each iteration
+        'iteration_timeout': 1.0,
     }
     
     # Bimodal-specific general settings defaults
@@ -53,19 +57,62 @@ class BimodalSemantics(SemanticDefaults):
     }
 
     def __init__(self, settings):
-
-        # Initialize the superclass to set defaults
+        # Initialize the superclass to set defaults and reset global state
         super().__init__(settings)
 
         # Initialize always true/false worlds, updated in the model_structure
         self.all_true = {}
         self.all_false = {}
+        
 
         # Initialize sorts, primitives, and frame_constraints
         self.define_sorts()
         self.define_primitives()
         self.frame_constraints = self.build_frame_constraints()
         self.premise_behavior, self.conclusion_behavior = self.define_invalidity()
+        
+    def _reset_global_state(self):
+        """Reset any global state that could cause interference between examples.
+        
+        This implementation ensures that each new instance of BimodalSemantics 
+        starts with a clean slate by resetting all shared resources that could
+        potentially cause interference between different examples.
+        
+        IMPORTANT: This method is critical for ensuring that examples run independently
+        and don't affect each other's results or performance. The BimodalSemantics
+        implementation demonstrates best practices for implementing this method in
+        theory-specific semantics classes:
+        
+        1. Call the parent implementation first using super()._reset_global_state()
+        2. Clear any theory-specific cache dictionaries
+        3. Reset mutable state while preserving necessary immutable definitions
+        4. Explicitly force garbage collection to release Z3 resources
+        
+        For more information, see theory_lib/notes/separation.md.
+        """
+        # Call parent implementation first
+        super()._reset_global_state()
+        
+        # Clear any cached world time intervals from previous examples
+        self.world_time_intervals = {}
+        
+        # Clear model cache values
+        if hasattr(self, 'model_structure'):
+            delattr(self, 'model_structure')
+        
+        # Reset mutable caches
+        self.all_true = {}
+        self.all_false = {}
+        
+        # Reset main point references (but not the definitions created in __init__)
+        self.main_world = 0
+        self.main_time = None
+        self.main_point = None
+        
+        # Force garbage collection to free any Z3 resources
+        import gc
+        gc.collect()
+        
 
     def define_sorts(self):
         """Define the Z3 sorts used in the bimodal logic model.
@@ -74,7 +121,7 @@ class BimodalSemantics(SemanticDefaults):
         - WorldStateSort: BitVecSort for representing world states as bitvectors
         - TimeSort: IntSort for representing time points
         - WorldIdSort: IntSort for mapping world IDs to world arrays
-        """
+        """            
         self.WorldStateSort = z3.BitVecSort(self.N)
         self.TimeSort = z3.IntSort()
         self.WorldIdSort = z3.IntSort()
@@ -328,12 +375,12 @@ class BimodalSemantics(SemanticDefaults):
             lawful,
             skolem_abundance,
             world_uniqueness,
-            time_interval,
+            # time_interval,
 
             # MAYBE
             # task_restriction,
             # task_minimization,
-            # world_interval,
+            world_interval,
         ]
 
     def is_valid_time(self, given_time, offset=0):
@@ -708,9 +755,10 @@ class BimodalSemantics(SemanticDefaults):
         These behaviors are used to find counterexamples that demonstrate invalidity of arguments
         by showing a case where all premises are true but the conclusion is false.
         """
-        # Use world ID directly instead of the world array
-        premise_behavior = lambda premise: self.true_at(premise, self.main_world, self.main_time)
-        conclusion_behavior = lambda conclusion: self.false_at(conclusion, self.main_world, self.main_time)
+        # Create main_point dictionary with world and time
+        main_point = {"world": self.main_world, "time": self.main_time}
+        premise_behavior = lambda premise: self.true_at(premise, main_point)
+        conclusion_behavior = lambda conclusion: self.false_at(conclusion, main_point)
         return premise_behavior, conclusion_behavior
         
     def verify_model(self, z3_model, premises, conclusions):
@@ -738,7 +786,8 @@ class BimodalSemantics(SemanticDefaults):
         # Check that all premises are true at the main point
         for premise in premises:
             try:
-                premise_expr = self.true_at(premise, self.main_world, self.main_time)
+                main_point = {"world": self.main_world, "time": self.main_time}
+                premise_expr = self.true_at(premise, main_point)
                 result = z3_model.eval(premise_expr)
                 if not z3.is_true(result):
                     verification_results["premises_verified"] = False
@@ -749,7 +798,8 @@ class BimodalSemantics(SemanticDefaults):
         # Check that all conclusions are false at the main point
         for conclusion in conclusions:
             try:
-                conclusion_expr = self.false_at(conclusion, self.main_world, self.main_time)
+                main_point = {"world": self.main_world, "time": self.main_time}
+                conclusion_expr = self.false_at(conclusion, main_point)
                 result = z3_model.eval(conclusion_expr)
                 if not z3.is_true(result):
                     verification_results["conclusions_verified"] = False
@@ -759,17 +809,22 @@ class BimodalSemantics(SemanticDefaults):
         
         return verification_results
 
-    def true_at(self, sentence, eval_world, eval_time):
-        """Returns a Z3 formula that is satisfied when the sentence is true at the given world at eval_time.
+    def true_at(self, sentence, eval_point):
+        """Returns a Z3 formula that is satisfied when the sentence is true at the given evaluation point.
 
         Args:
             sentence: The sentence to evaluate
-            eval_world: The world ID (integer) at which to evaluate the sentence
-            eval_time: The time point at which to evaluate the sentence
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID (integer) at which to evaluate the sentence
+                - "time": The time point at which to evaluate the sentence
             
         Returns:
-            Z3 formula that is satisfied when sentence is true at eval_world at eval_time
+            Z3 formula that is satisfied when sentence is true at eval_point
         """
+        # Extract world and time from eval_point
+        eval_world = eval_point["world"]
+        eval_time = eval_point["time"]
+        
         # Get the world array from the world ID
         world_array = self.world_function(eval_world)
         
@@ -783,20 +838,21 @@ class BimodalSemantics(SemanticDefaults):
         # recursive case
         operator = sentence.operator  # store operator
         arguments = sentence.arguments or () # store arguments
-        return operator.true_at(*arguments, eval_world, eval_time) # apply semantics
+        return operator.true_at(*arguments, eval_point) # apply semantics
 
-    def false_at(self, sentence, eval_world, eval_time):
-        """Returns a Z3 formula that is satisfied when the sentence is false at eval_world at eval_time.
+    def false_at(self, sentence, eval_point):
+        """Returns a Z3 formula that is satisfied when the sentence is false at the given evaluation point.
 
         Args:
             sentence: The sentence to evaluate
-            eval_world: The world ID at which to evaluate the sentence
-            eval_time: The time point at which to evaluate the sentence
+            eval_point: Dictionary containing evaluation parameters:
+                - "world": The world ID at which to evaluate the sentence
+                - "time": The time point at which to evaluate the sentence
             
         Returns:
-            Z3 formula that is satisfied when sentence is false at eval_world at eval_time
+            Z3 formula that is satisfied when sentence is false at eval_point
         """
-        return z3.Not(self.true_at(sentence, eval_world, eval_time))
+        return z3.Not(self.true_at(sentence, eval_point))
 
     def generate_time_intervals(self, M):
         """Generate all valid time intervals of length M that include time 0.
@@ -1144,10 +1200,10 @@ class BimodalSemantics(SemanticDefaults):
                                     time_shift_relations[source_id][shift] = target_id
                                     break
                         except Exception as e:
-                            print(f"Warning: Error checking time-shift relation for worlds {source_id}->{target_id}: {e}")
+                            pass
         
         if not world_histories:
-            print("Warning: No valid world histories found in the model!")
+            pass
             
         return time_shift_relations
 
@@ -1203,8 +1259,9 @@ class BimodalProposition(PropositionDefaults):
         # Calculate the extension (truth/falsity at each world and time)
         self.extension = self.find_extension()
         
+        # TODO: adapt find_truth_condition in operators.py to use eval_point
         # Extract world states sets for use in representation
-        self.truth_set, self.false_set = self.extract_world_states()
+        self.truth_set, self.false_set = self._find_proposition_at(self.eval_time)
 
     def __eq__(self, other):
         return (
@@ -1213,55 +1270,7 @@ class BimodalProposition(PropositionDefaults):
         )
 
     def __repr__(self):
-        z3_model = self.model_structure.z3_model
-        
-        # Handle truth set
-        truth_worlds = set()
-        for world_state in self.truth_set:
-            # If it's already a string representation, use it directly
-            if isinstance(world_state, str) and not world_state.startswith('<unknown-'):
-                truth_worlds.add(world_state)
-            # If it's a Z3 BitVec
-            elif hasattr(world_state, 'as_ast'):
-                evaluated_bit = z3_model.evaluate(world_state)
-                truth_worlds.add(bitvec_to_worldstate(evaluated_bit))
-            # If it's any other type
-            else:
-                try:
-                    # Try to convert it to a world state
-                    if hasattr(world_state, 'as_long') or isinstance(world_state, int):
-                        truth_worlds.add(bitvec_to_worldstate(world_state))
-                    else:
-                        # Fall back to using the string representation
-                        truth_worlds.add(str(world_state))
-                except Exception:
-                    # If all else fails, use the string representation
-                    truth_worlds.add(str(world_state))
-                
-        # Handle false set
-        false_worlds = set()
-        for world_state in self.false_set:
-            # If it's already a string representation, use it directly
-            if isinstance(world_state, str) and not world_state.startswith('<unknown-'):
-                false_worlds.add(world_state)
-            # If it's a Z3 BitVec
-            elif hasattr(world_state, 'as_ast'):
-                evaluated_bit = z3_model.evaluate(world_state)
-                false_worlds.add(bitvec_to_worldstate(evaluated_bit))
-            # If it's any other type
-            else:
-                try:
-                    # Try to convert it to a world state
-                    if hasattr(world_state, 'as_long') or isinstance(world_state, int):
-                        false_worlds.add(bitvec_to_worldstate(world_state))
-                    else:
-                        # Fall back to using the string representation
-                        false_worlds.add(str(world_state))
-                except Exception:
-                    # If all else fails, use the string representation
-                    false_worlds.add(str(world_state))
-                
-        return f"< {pretty_set_print(truth_worlds)}, {pretty_set_print(false_worlds)} >"
+        return f"< {pretty_set_print(self.truth_set)}, {pretty_set_print(self.false_set)} >"
 
     def proposition_constraints(self, sentence_letter):
         """Returns Z3 constraints for a sentence letter based on user settings.
@@ -1350,7 +1359,7 @@ class BimodalProposition(PropositionDefaults):
                     # Pass the world_id directly to the true_at method
                     # Allow Z3 exceptions to propagate naturally - fail fast
                     truth_expr = self.model_structure.semantics.true_at(
-                        self.sentence, world_id, time
+                        self.sentence, {"world" : world_id, "time" : time}
                     )
                     evaluated_expr = self.z3_model.evaluate(truth_expr)
                     if z3.is_true(evaluated_expr):
@@ -1365,100 +1374,12 @@ class BimodalProposition(PropositionDefaults):
             
         elif self.operator is not None:
             # For complex sentences, delegate to the operator's find_truth_condition method
-            # Consistently use world ID
-            return self.operator.find_truth_condition(*arguments, self.eval_world, self.eval_time)
+            # Create an eval_point dictionary to pass world and time consistently
+            eval_point = {"world": self.eval_world, "time": self.eval_time}
+            return self.operator.find_truth_condition(*arguments, eval_point)
             
         raise ValueError(f"There is no proposition for {self}.")
 
-    def extract_world_states(self) -> tuple[set, set]:
-        """Extract sets of world states where the proposition is true and false.
-        
-        Processes the proposition's temporal profiles for each world to determine
-        the set of world states where the proposition is true and where it is false.
-        
-        Returns:
-            tuple[set, set]: A pair of sets containing:
-                - First set: World states where the proposition is true
-                - Second set: World states where the proposition is false
-                
-        Raises:
-            KeyError: If a required world ID is missing
-            ValueError: If world state extraction fails in a way that breaks the model
-        """
-        # Extract states where proposition is true
-        truth_states = set()
-        false_states = set()
-        
-        for world_id, (true_times, false_times) in self.extension.items():
-            # Get the world array for this world ID - fail if not found
-            if world_id not in self.model_structure.world_arrays:
-                raise KeyError(f"World {world_id} not in world_arrays, but required for proposition {self.name}")
-                
-            world_array = self.model_structure.world_arrays[world_id]
-            
-            # Prefer world histories when available
-            if world_id in self.model_structure.world_histories:
-                world_history = self.model_structure.world_histories[world_id]
-                
-                # Process true times
-                for time in true_times:
-                    if time not in world_history:
-                        # Skip times that aren't in the world history instead of raising an error
-                        # This handles cases where time intervals don't perfectly match
-                        continue
-                        
-                    state = world_history[time]
-                    # Add the state to the truth set
-                    if isinstance(state, str):
-                        truth_states.add(state)
-                    else:
-                        # Convert to string representation if needed
-                        state_str = str(state)
-                        truth_states.add(state_str)
-                
-                # Process false times
-                for time in false_times:
-                    if time not in world_history:
-                        # Skip times that aren't in the world history instead of raising an error
-                        continue
-                        
-                    state = world_history[time]
-                    # Add the state to the false set
-                    if isinstance(state, str):
-                        false_states.add(state)
-                    else:
-                        # Convert to string representation if needed
-                        state_str = str(state)
-                        false_states.add(state_str)
-            else:
-                # Direct array access using safe_select
-                
-                # Process true times
-                for time in true_times:
-                    try:
-                        # Use safe_select with direct time value - it handles type conversion
-                        state = self.model_structure.semantics.safe_select(
-                            self.z3_model, world_array, time)
-                        state_val = bitvec_to_worldstate(state)
-                        truth_states.add(state_val)
-                    except (TypeError, z3.Z3Exception) as e:
-                        error_msg = f"Failed to extract state at world {world_id}, time {time}: {e}"
-                        raise ValueError(error_msg) from e
-                
-                # Process false times
-                for time in false_times:
-                    try:
-                        # Use safe_select with direct time value - it handles type conversion
-                        state = self.model_structure.semantics.safe_select(
-                            self.z3_model, world_array, time)
-                        state_val = bitvec_to_worldstate(state)
-                        false_states.add(state_val)
-                    except (TypeError, z3.Z3Exception) as e:
-                        error_msg = f"Failed to extract state at world {world_id}, time {time}: {e}"
-                        raise ValueError(error_msg) from e
-                
-        return truth_states, false_states
-
     def truth_value_at(self, eval_world, eval_time):
         """Checks if the proposition is true at the given world and time.
         
@@ -1475,12 +1396,12 @@ class BimodalProposition(PropositionDefaults):
         # Check if we have a valid extension
         if not hasattr(self, 'extension') or not self.extension:
             # If there's no extension, we can't evaluate truth
-            print(f"WARNING: No extension available for proposition {self} at world {eval_world}")
+            pass
             return False
             
         # Check if the requested world_id exists in the extension
         if eval_world not in self.extension:
-            print(f"WARNING: World ID {eval_world} not found in extension for {self}")
+            pass
             # Return a default value when the world doesn't exist in the extension
             return False
             
@@ -1492,18 +1413,58 @@ class BimodalProposition(PropositionDefaults):
         
         if true_in_eval_world and false_in_eval_world:
             # Both true and false (shouldn't happen in a well-formed model)
+            # TODO: make print world_history instead
             try:
                 world_array = self.model_structure.get_world_array(eval_world)
                 eval_state = self.z3_model.evaluate(world_array[eval_time])
-                print(
-                    f"WARNING: the world_id {eval_world} at time {eval_time} "
-                    f"makes {self} both true and false at the world_state {eval_state}."
-                )
+                pass
             except Exception as e:
-                print(f"WARNING: Error accessing world state: {e}")
+                pass
                 
         return true_in_eval_world
 
+    def _find_proposition_at(self, eval_time):
+        """Find the proposition's extension at a specific evaluation point.
+        
+        This method determines which world states make the proposition true and false
+        at the specified evaluation time by examining the proposition's extension
+        across all worlds.
+        
+        Args:
+            eval_point (dict): Dictionary containing evaluation information with keys:
+                - time (int): The time point at which to evaluate
+                - world (int): The world ID (not used in this method since we collect
+                             states from all worlds at the given time)
+                
+        Returns:
+            str: A string representation of the proposition's extension at the evaluation
+                 point in the format "< truth_states, false_states >" where:
+                 - truth_states: Set of world states where proposition is true at eval_time
+                 - false_states: Set of world states where proposition is false at eval_time
+        """
+        # Initialize sets to collect world states where proposition is true/false
+        truth_states = set()
+        false_states = set()
+
+        # Examine each world's extension at the evaluation time
+        for world_id, (true_times, false_times) in self.extension.items():
+            # Get the world history containing time->state mappings
+            world_history = self.model_structure.world_histories[world_id]
+
+            # If eval_time is in true_times, add the corresponding state to truth_states
+            if eval_time in true_times:
+                state = world_history[eval_time]
+                truth_states.add(state)
+                
+            # If eval_time is in false_times, add the corresponding state to false_states
+            if eval_time in false_times:
+                state = world_history[eval_time]
+                false_states.add(state)
+                
+        # Return proposition's extension at eval_time
+        return truth_states, false_states
+
+    # TODO: make print from operator truth_condition
     def print_proposition(self, eval_point, indent_num, use_colors):
         """Print the proposition and it's truth value at the evaluation point.
         
@@ -1548,7 +1509,7 @@ class BimodalProposition(PropositionDefaults):
             world_state_repr,
             use_colors
         )
-        
+
         # Print the proposition
         print(
             f"{'  ' * indent_num}{FULL}|{self.name}| = {self}{RESET}"
@@ -1579,9 +1540,16 @@ class BimodalStructure(ModelDefaults):
             model_constraints (ModelConstraints): Constraints for model generation
             max_time (int): Maximum solving time in seconds
         """
+        # Explicitly reset any Z3 resources before initializing
+        import gc
+        gc.collect()
+
         # Initialize parent class first
         super().__init__(model_constraints, max_time)
 
+        # We don't want to reset semantics again here as it would 
+        # remove necessary attributes already defined in __init__
+
         # Initialize temporal and world attributes
         self.M = self.semantics.M
         # Update time range to be centered around 0: [-M+1, M-1]
@@ -1598,6 +1566,9 @@ class BimodalStructure(ModelDefaults):
         self.z3_main_world_state = None
         # Initialize main_time with a default value (0) to avoid AttributeError
         self.main_time = 0
+        
+        # Force another garbage collection to ensure no Z3 resources leak
+        gc.collect()
 
         # Only evaluate if we have a valid model
         if self.z3_model_status and self.z3_model is not None:
